import { consoleError, consoleLog, createJsonBuffer, getAllFiles, getRootPath, getTranslationKeys, } from "./utils.js";
import fs from "fs";
import path from "path";
import { typeis, DataTypes } from "typeof-plus";
import { getTranslatedMap } from "./translate.js";
export const analysis = async (config) => {
    var _a, _b, _c;
    if (!config.entrys || !config.output) {
        throw new Error("缺少 'entrys' 或者 'output' 配置项");
    }
    if (typeis(config.entrys) !== DataTypes.array) {
        throw new Error("'entrys' 配置项必须为 Array<string>");
    }
    if (typeis(config.output) !== DataTypes.string) {
        throw new Error("'output' 配置项必须为 string");
    }
    if (!config.translateConfig) {
        throw new Error("缺少 'translateConfig' 配置项");
    }
    if (!((_a = config.translateConfig) === null || _a === void 0 ? void 0 : _a.proxy)) {
        throw new Error("'translateConfig' 中 缺少 'proxy' 配置项");
    }
    if (!((_b = config.translateConfig) === null || _b === void 0 ? void 0 : _b.to)) {
        throw new Error("'translateConfig' 中 缺少 'to' 配置项");
    }
    if (!((_c = config.translateConfig) === null || _c === void 0 ? void 0 : _c.fromLang)) {
        throw new Error("'translateConfig' 中 缺少 'fromLang' 配置项");
    }
    consoleLog("开始分析 i18n namespaces 和 keys");
    const files = getAllFiles(config.entrys);
    const fragmentMap = [];
    files.forEach((file) => {
        const namespaces = [];
        file.replace(/useTranslation[\s]*\((.+)\)/g, (s, namespace, index) => {
            namespace = namespace.replace(/['"`\s]/g, "");
            namespaces.push({
                namespace,
                index,
            });
            return "";
        });
        if (namespaces.length === 0) {
            return;
        }
        const fragments = namespaces.map((item, key) => {
            var _a, _b;
            return {
                namespace: item.namespace,
                fragment: file.slice(item.index, (_b = (_a = namespaces[key + 1]) === null || _a === void 0 ? void 0 : _a.index) !== null && _b !== void 0 ? _b : file.length),
            };
        });
        fragmentMap.push(...fragments);
    });
    const namespaceMap = {};
    fragmentMap.forEach(({ namespace, fragment }) => {
        const namespaceItem = namespaceMap[namespace];
        const keys = getTranslationKeys(fragment);
        keys.forEach((key) => {
            if (namespaceItem) {
                namespaceItem[key] = key;
            }
            else {
                namespaceMap[namespace] = { [key]: key };
            }
        });
    });
    consoleLog("i18n namespaces 和 keys 分析完成,等待生成配置...");
    Object.entries(namespaceMap).forEach(([namespace, val]) => {
        const outDir = path.join(getRootPath(), config.output, `${namespace}.json`);
        fs.writeFile(outDir, createJsonBuffer(val), (err) => {
            if (err) {
                console.error(err);
                throw new Error("写入配置出错");
            }
            consoleLog(`配置生成成功 --> ${outDir}`);
        });
    });
    const recordMap = JSON.parse(JSON.stringify(namespaceMap));
    const recordOutputDir = path.join(getRootPath(), "autotranslation.record.json");
    const delMap = {};
    const addMap = {};
    const delDirList = [];
    let recordJson = {};
    try {
        recordJson = JSON.parse(fs.readFileSync(recordOutputDir, "utf8"));
    }
    catch (error) { }
    for (const namespace in recordMap) {
        const val = recordMap[namespace];
        recordMap[namespace] = Object.keys(val);
        // --- 处理 addMap ---
        if (recordJson[namespace]) {
            recordMap[namespace].forEach((key) => {
                if (!recordJson[namespace].includes(key)) {
                    if (addMap[namespace]) {
                        addMap[namespace].push(key);
                    }
                    else {
                        addMap[namespace] = [key];
                    }
                }
            });
        }
        else {
            addMap[namespace] = recordMap[namespace];
        }
    }
    // --- 处理 delMap/delDirList ---
    for (const namespace in recordJson) {
        const keys = recordJson[namespace];
        if (recordMap[namespace]) {
            keys.forEach((key) => {
                if (!recordMap[namespace].includes(key)) {
                    if (delMap[namespace]) {
                        delMap[namespace].push(key);
                    }
                    else {
                        delMap[namespace] = [key];
                    }
                }
            });
        }
        else {
            delDirList.push(namespace);
        }
    }
    consoleLog("变动分析完成");
    consoleLog("↓↓↓↓↓↓↓↓↓↓");
    consoleLog("待删除 keys Map: ", delMap);
    consoleLog("待新增 keys Map: ", addMap);
    consoleLog("待删除 namespace 文件: ", delDirList);
    consoleLog("翻译已启动,请耐心等待...");
    // 开始翻译
    const translateRun = () => new Promise((resolve) => {
        const startTime = new Date().getTime();
        let count = config.translateConfig.to.length;
        config.translateConfig.to.forEach(async (item) => {
            const targetMap = await getTranslatedMap({
                proxy: config.translateConfig.proxy,
                fromLang: config.translateConfig.fromLang,
                targetLang: item.targetLang,
                fromMap: addMap,
            });
            for (const namespace in targetMap) {
                const tempPath = path.join(getRootPath(), item.outPath, `${namespace}.json`);
                let tempNamespaceJson = {};
                try {
                    tempNamespaceJson = JSON.parse(fs.readFileSync(tempPath, "utf8"));
                }
                catch (error) { }
                for (const key in targetMap[namespace]) {
                    tempNamespaceJson[key] = targetMap[namespace][key];
                }
                fs.writeFileSync(tempPath, createJsonBuffer(tempNamespaceJson));
                consoleLog(`增量翻译更新 keys 成功 --> ${tempPath}`);
            }
            if (--count <= 0) {
                consoleLog(`翻译完成,耗时: ${Number((new Date().getTime() - startTime) / 1000)}s`);
                resolve(true);
            }
        });
    });
    await translateRun();
    // 处理已删除的 namespace
    delDirList.forEach((namespace) => {
        const outDir = path.join(getRootPath(), config.output, `${namespace}.json`);
        fs.unlink(outDir, (err) => {
            if (err) {
                console.error(err);
                return;
            }
            consoleLog(`已清理失效配置 --> ${outDir}`);
        });
        config.translateConfig.to.forEach((item) => {
            const outDir_to = path.join(getRootPath(), item.outPath, `${namespace}.json`);
            fs.unlink(outDir_to, (err) => {
                if (err) {
                    return;
                }
                consoleLog(`已清理失效配置 --> ${outDir_to}`);
            });
        });
    });
    // 处理已删除的 keys
    config.translateConfig.to.forEach((item) => {
        for (const namespace in delMap) {
            const keys = delMap[namespace];
            const tempPath = path.join(getRootPath(), item.outPath, `${namespace}.json`);
            let tempNamespaceJson = {};
            try {
                tempNamespaceJson = JSON.parse(fs.readFileSync(tempPath, "utf8"));
            }
            catch (error) {
                consoleError(error);
                throw new Error("清理失效 keys 读取 json 出错");
            }
            keys.forEach((key) => {
                delete tempNamespaceJson[key];
            });
            fs.writeFile(tempPath, createJsonBuffer(tempNamespaceJson), (err) => {
                if (err) {
                    consoleError(err);
                    throw new Error("清理失效 keys 写入 json 出错");
                }
                consoleLog(`清理失效 keys 成功 --> ${tempPath}`);
            });
        }
    });
    // 翻译完最后写入记录避免翻译失败记录错误
    fs.writeFile(recordOutputDir, JSON.stringify(recordMap), (err) => {
        if (err) {
            consoleError("写入分析记录出错");
            console.error(err);
            return;
        }
        consoleLog(`分析记录已更新 --> ${recordOutputDir}`);
    });
};
