import translate from "@vitalets/google-translate-api";
import tunnel from "tunnel";
import { consoleError, consoleSuccess, consoleWarn } from "./utils.js";
export const getTranslatedMap = async ({ proxy, fromLang, targetLang, fromMap, }) => {
    const translator = (key) => new Promise((resolve, reject) => {
        let failedNum = 0;
        const run = () => {
            translate(key, { from: fromLang, to: targetLang }, {
                agent: tunnel.httpsOverHttp({
                    proxy: {
                        host: proxy.ip,
                        port: proxy.port,
                        headers: {
                            "User-Agent": "Node",
                        },
                    },
                }),
            })
                .then(({ text }) => {
                resolve(text);
            })
                .catch((err) => {
                if (++failedNum > 10) {
                    consoleError("重试超过10次,程序终止,请检查网络代理后重试");
                    reject(err);
                }
                else {
                    consoleWarn("翻译失败,正在重试...");
                    setTimeout(() => {
                        run();
                    }, 1000);
                }
            });
        };
        run();
    });
    const targetMap = {};
    for (const namespace in fromMap) {
        targetMap[namespace] = {};
        const keys = fromMap[namespace];
        for (const key of keys) {
            let resText = "";
            if (/\{\{.+?\}\}/.test(key)) {
                const texts = key.split(/\{\{.+?\}\}/g);
                const slots = [];
                key.replace(/\{\{.+?\}\}/g, (v) => {
                    slots.push(v);
                    return "";
                });
                for (let i = 0; i < texts.length; i++) {
                    const key = texts[i];
                    if (key) {
                        const resFragment = await translator(key);
                        texts[i] = resFragment;
                    }
                }
                for (let j = 0; j < texts.length; j++) {
                    const str = texts[j];
                    resText += str;
                    if (j < texts.length - 1) {
                        resText += slots[j];
                    }
                }
            }
            else {
                resText = await translator(key);
            }
            consoleSuccess(`${fromLang}: ${key} ---> ${targetLang}: ${resText}`);
            targetMap[namespace][key] = resText;
        }
    }
    return targetMap;
};
