import fs from "fs";
import path from "path";
const cwd = process.cwd();
export const getRootPath = () => cwd;
export const defineConfig = (config) => {
    return config;
};
export const getFile = (filePath) => {
    return fs.readFileSync(filePath, "utf8");
};
export const getFiles = (entry, deep) => {
    const result = [];
    const readDir = (entry) => {
        const dirInfo = fs.readdirSync(entry);
        for (let j = 0; j < dirInfo.length; j++) {
            const item = dirInfo[j];
            const location = path.join(entry, item);
            const info = fs.statSync(location);
            if (info.isDirectory()) {
                deep && readDir(location);
            }
            else {
                result.push(getFile(location));
            }
        }
    };
    readDir(entry);
    return result;
};
export const getAllFiles = (entrys) => {
    const files = [];
    entrys.forEach((item) => {
        let fileMap = null;
        item.replace(/([^\*]+)\/\*$/, (s, directory) => {
            fileMap = getFiles(path.join(getRootPath(), directory), false);
            return "";
        });
        item.replace(/([^\*]+)\/\*\*\/\*$/, (s, deepDirectory) => {
            fileMap = getFiles(path.join(getRootPath(), deepDirectory), true);
            return "";
        });
        if (fileMap === null) {
            fileMap = [getFile(path.join(getRootPath(), item))];
        }
        files.push(...fileMap);
    });
    return files;
};
export const getTranslationKeys = (fragment) => {
    const keys = [];
    fragment.replace(/(?<!\w)t\s*\(((`[\w\W]+`.*)|.+)\)/g, (v) => {
        v.replace(/(?<=\()('.+?'|".+?"|`[\w\W]+?`)/, (v) => {
            keys.push(v.slice(1, -1));
            return "";
        });
        return "";
    });
    return keys;
};
export const createJsonBuffer = (val) => {
    let outputBuffer = "{\n";
    for (const textKey in val) {
        outputBuffer += `\t${JSON.stringify({ [textKey]: val[textKey] }).slice(1, -1)},\n`;
    }
    outputBuffer = outputBuffer.slice(0, -2);
    outputBuffer += "\n}\n";
    return outputBuffer;
};
export const consoleSuccess = (...msg) => console.log("\x1B[32m%s\x1B[0m", ...msg);
export const consoleLog = (...msg) => console.log("\x1B[36m%s\x1B[0m", ...msg);
export const consoleWarn = (...msg) => console.log("\x1B[33m%s\x1B[0m", ...msg);
export const consoleError = (...msg) => console.log("\x1B[31m%s\x1B[0m", ...msg);
